==> react-script-1.0.25.tgz: package/lib/webpack.config.js <==
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// const Config = require('webpack-chain')
// const config = new Config()

module.exports = ({ config, paths }) => {
  const MODE_DEV = process.env.APP_MODE === 'development'
  const MODE_PROD = process.env.APP_MODE === 'production'

  const genUrlLoaderOptions = (dir) => {
    return {
      limit: 1024 * 8,
      name: MODE_PROD
        ? `${paths.assetsSubPath}${dir}[name].[contenthash:8].[ext]`
        : '[name].[ext]',
      esModule: false
    }
  }

  /**
   * webpack 基本配置
   */
  config
    .mode(process.env.APP_MODE)
    .devtool(MODE_PROD ? undefined : 'cheap-module-eval-source-map')
    .entry('main')
    .add(paths.appEntry)
    .end()
    .output.path(MODE_PROD ? paths.appBuild : undefined)
    .filename(
      MODE_PROD
        ? `${paths.assetsSubPath}${paths.assetsJsPath}[name].[contenthash:8].js`
        : '[name].js'
    )
    .publicPath(paths.publicPath)
    .end()
    .resolve.alias.set('@', paths.appSrc)
    .end()
    .extensions.merge(['.ts', '.tsx', '.js'])

  /**
   * webpack loader 配置
   */
  config.module
    .rule('babel')
    .test(/\.(j|t)sx?$/)
    .include.add(paths.appSrc)
    .end()
    .use('babel')
    .loader('babel-loader')
    .options({
      presets: [
        [
          '@babel/preset-env',
          {
            modules: false
          }
        ],
        '@babel/preset-react',
        '@babel/preset-typescript'
      ],
      plugins: [
        [
          '@babel/plugin-transform-runtime',
          {
            corejs: {
              version: 3,
              proposals: true
            },
            useESModules: true
          }
        ],
        '@babel/plugin-syntax-dynamic-import',
        ['@babel/plugin-proposal-decorators', { legacy: true }],
        ['@babel/plugin-proposal-class-properties', { loose: true }],
        [
          'import',
          {
            libraryName: 'antd',
            libraryDirectory: 'es',
            style: true
          }
        ]
      ]
    })
    .end()
    .end()
    .rule('style')
    .test(/\.(c|le)ss$/)
    .use('style')
    .loader(MODE_PROD ? MiniCssExtractPlugin.loader : 'style-loader')
    .end()
    .use('css')
    .loader('css-loader')
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options({
      plugins: [require('autoprefixer')({})]
    })
    .end()
    .use('less')
    .loader('less-loader')
    .options({
      lessOptions: {
        javascriptEnabled: true,
        modifyVars: {}
      }
    })
    .end()
    .end()
    .rule('image')
    .test(/\.(png|svg|jpe?g|gif|webp)(\?.*)?$/)
    .use('image')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsImgPath))
    .end()
    .end()
    .rule('font')
    .test(/\.(woff2?|eot|ttf|otf)(\?.*)?$/i)
    .use('font')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsFontPath))
    .end()
    .end()
    .rule('media')
    .test(/\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/)
    .use('media')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsMediaPath))

  config.when(MODE_DEV, (config) => {
    config.module
      .rule('babel')
      .use('babel')
      .tap((options) => {
        options.plugins.unshift('react-refresh/babel')
        return options
      })
  })

  /**
   * webpack plugins 配置
   */
  config
    .plugin('env')
    .use(require.resolve('dotenv-webpack'), [
      {
        path: `.env.${process.env.APP_ENV}`
      }
    ])
    .end()
    .plugin('html')
    .use(require.resolve('html-webpack-plugin'), [
      {
        template: paths.appTemplate,
        filename: 'index.html'
      }
    ])

  config
    .when(MODE_DEV, (config) => {
      config
        .plugin('error')
        .use(require.resolve('friendly-errors-webpack-plugin'))
        .end()
        .plugin('refresh')
        .use(require.resolve('@pmmmwh/react-refresh-webpack-plugin'))
    })
    .when(MODE_PROD, (config) => {
      config
        .plugin('css-extract')
        .use(MiniCssExtractPlugin, [
          {
            filename: `${paths.assetsSubPath}${paths.assetsCssPath}[name].[contenthash:8].css`
          }
        ])
        .end()
        .plugin('copy')
        .use(require.resolve('copy-webpack-plugin'), [
          {
            patterns: [
              {
                from: paths.appStatic,
                to: paths.appBuild,
                globOptions: {
                  ignore: ['index.html']
                }
              }
            ]
          }
        ])
        .end()
        .plugin('progress')
        .use(require.resolve('webpackbar'), [
          {
            name: '开始构建项目'
          }
        ])
    })

  /**
   * webpack dev server 配置
   */
  if (MODE_DEV) {
    config.devServer
      .port(4000)
      .host('0.0.0.0')
      .hot(true)
      .quiet(true)
      .historyApiFallback(true)
      .clientLogLevel('none')
  }

  /**
   * webpack 打包配置
   */
  if (MODE_PROD) {
    config.performance
      .hints(false)
      .end()
      .optimization.runtimeChunk({
        name: 'runtime'
      })
      .minimizer('minicss')
      .use(require.resolve('optimize-css-assets-webpack-plugin'))
      .end()
      .minimizer('minijs')
      .use(require.resolve('terser-webpack-plugin'), [
        {
          parallel: true,
          extractComments: true
        }
      ])
      .end()
      .splitChunks({
        chunks: 'all',
        cacheGroups: {
          vendors: {
            name: 'vendor',
            test: /[\\/]node_modules[\\/]/,
            priority: -10,
            chunks: 'initial'
          },
          common: {
            name: 'common',
            minChunks: 2,
            priority: -20,
            chunks: 'initial'
          }
        }
      })
  }
}

==> react-script-1.0.27.tgz: package/lib/webpack.config.js <==
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// const Config = require('webpack-chain')
// const config = new Config()

module.exports = ({ config, paths }) => {
  const MODE_DEV = process.env.APP_MODE === 'development'
  const MODE_PROD = process.env.APP_MODE === 'production'

  const genUrlLoaderOptions = (dir) => {
    return {
      limit: 1024 * 8,
      name: MODE_PROD
        ? `${paths.assetsSubPath}${dir}[name].[contenthash:8].[ext]`
        : '[name].[ext]',
      esModule: false
    }
  }

  /**
   * webpack 基本配置
   */
  config
    .mode(process.env.APP_MODE)
    .devtool(MODE_PROD ? undefined : 'cheap-module-eval-source-map')
    .entry('main')
    .add(paths.appEntry)
    .end()
    .output.path(MODE_PROD ? paths.appBuild : undefined)
    .filename(
      MODE_PROD
        ? `${paths.assetsSubPath}${paths.assetsJsPath}[name].[contenthash:8].js`
        : '[name].js'
    )
    .publicPath(paths.publicPath)
    .end()
    .resolve.alias.set('@', paths.appSrc)
    .end()
    .extensions.merge(['.ts', '.tsx', '.js'])

  /**
   * webpack loader 配置
   */
  config.module
    .rule('babel')
    .test(/\.(j|t)sx?$/)
    .include.add(paths.appSrc)
    .end()
    .use('babel')
    .loader('babel-loader')
    .options({
      presets: [
        [
          '@babel/preset-env',
          {
            modules: false
          }
        ],
        '@babel/preset-react',
        '@babel/preset-typescript',
        'linaria/babel'
      ],
      plugins: [
        [
          '@babel/plugin-transform-runtime',
          {
            corejs: {
              version: 3,
              proposals: true
            },
            useESModules: true
          }
        ],
        '@babel/plugin-syntax-dynamic-import',
        ['@babel/plugin-proposal-decorators', { legacy: true }],
        ['@babel/plugin-proposal-class-properties', { loose: true }],
        [
          'import',
          {
            libraryName: 'antd',
            libraryDirectory: 'es',
            style: true
          }
        ]
      ]
    })
    .end()
    .use('linaria')
    .loader('linaria/loader')
    .options({
      sourceMap: MODE_DEV
    })
    .end()
    .rule('style')
    .test(/\.(c|le)ss$/)
    .use('style')
    .loader(MiniCssExtractPlugin.loader)
    .options({
      hmr: MODE_DEV
    })
    .end()
    .use('css')
    .loader('css-loader')
    .options({
      sourceMap: MODE_DEV
    })
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options({
      plugins: [require('autoprefixer')({})]
    })
    .end()
    .use('less')
    .loader('less-loader')
    .options({
      lessOptions: {
        javascriptEnabled: true,
        modifyVars: {}
      }
    })
    .end()
    .end()
    .rule('image')
    .test(/\.(png|svg|jpe?g|gif|webp)(\?.*)?$/)
    .use('image')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsImgPath))
    .end()
    .end()
    .rule('font')
    .test(/\.(woff2?|eot|ttf|otf)(\?.*)?$/i)
    .use('font')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsFontPath))
    .end()
    .end()
    .rule('media')
    .test(/\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/)
    .use('media')
    .loader('url-loader')
    .options(genUrlLoaderOptions(paths.assetsMediaPath))

  config.when(MODE_DEV, (config) => {
    config.module
      .rule('babel')
      .use('babel')
      .tap((options) => {
        options.plugins.unshift('react-refresh/babel')
        return options
      })
  })

  /**
   * webpack plugins 配置
   */
  config
    .plugin('env')
    .use(require.resolve('dotenv-webpack'), [
      {
        path: `.env.${process.env.APP_ENV}`
      }
    ])
    .end()
    .plugin('html')
    .use(require.resolve('html-webpack-plugin'), [
      {
        template: paths.appTemplate,
        filename: 'index.html'
      }
    ])

  config
    .when(MODE_DEV, (config) => {
      config
        .plugin('error')
        .use(require.resolve('friendly-errors-webpack-plugin'))
        .end()
        .plugin('refresh')
        .use(require.resolve('@pmmmwh/react-refresh-webpack-plugin'))
    })
    .when(MODE_PROD, (config) => {
      config
        .plugin('css-extract')
        .use(MiniCssExtractPlugin, [
          {
            filename: `${paths.assetsSubPath}${paths.assetsCssPath}[name].[contenthash:8].css`
          }
        ])
        .end()
        .plugin('copy')
        .use(require.resolve('copy-webpack-plugin'), [
          {
            patterns: [
              {
                from: paths.appStatic,
                to: paths.appBuild,
                globOptions: {
                  ignore: ['index.html']
                }
              }
            ]
          }
        ])
        .end()
        .plugin('progress')
        .use(require.resolve('webpackbar'), [
          {
            name: '开始构建项目'
          }
        ])
    })

  /**
   * webpack dev server 配置
   */
  if (MODE_DEV) {
    config.devServer
      .port(4000)
      .host('0.0.0.0')
      .hot(true)
      .quiet(true)
      .historyApiFallback(true)
      .clientLogLevel('none')
  }

  /**
   * webpack 打包配置
   */
  if (MODE_PROD) {
    config.performance
      .hints(false)
      .end()
      .optimization.runtimeChunk({
        name: 'runtime'
      })
      .minimizer('minicss')
      .use(require.resolve('optimize-css-assets-webpack-plugin'))
      .end()
      .minimizer('minijs')
      .use(require.resolve('terser-webpack-plugin'), [
        {
          parallel: true,
          extractComments: true
        }
      ])
      .end()
      .splitChunks({
        chunks: 'all',
        cacheGroups: {
          vendors: {
            name: 'vendor',
            test: /[\\/]node_modules[\\/]/,
            priority: -10,
            chunks: 'initial'
          },
          common: {
            name: 'common',
            minChunks: 2,
            priority: -20,
            chunks: 'initial'
          }
        }
      })
  }
}
